@@ -20,7 +20,6 @@ import warnings
 
 from girder_jobs.constants import JobStatus
 from girder_jobs.models.job import Job
-from pkg_resources import DistributionNotFound, get_distribution
 
 import girder
 import large_image
@@ -28,6 +27,7 @@ from girder import events, logger
 from girder.constants import AccessType
 from girder.exceptions import ValidationException
 from girder.models.file import File
+from girder.models.folder import Folder
 from girder.models.item import Item
 from girder.models.notification import Notification
 from girder.models.setting import Setting
@@ -40,16 +40,26 @@ from . import constants, girder_tilesource
 from .girder_tilesource import getGirderTileSource  # noqa
 from .loadmodelcache import invalidateLoadModelCache
 from .models.image_item import ImageItem
+from .rest import addSystemEndpoints
 from .rest.large_image_resource import LargeImageResource
 from .rest.tiles import TilesItemResource
 
 try:
-    __version__ = get_distribution(__name__).version
-except DistributionNotFound:
+    from importlib.metadata import PackageNotFoundError
+    from importlib.metadata import version as _importlib_version
+except ImportError:
+    from importlib_metadata import PackageNotFoundError
+    from importlib_metadata import version as _importlib_version
+try:
+    __version__ = _importlib_version(__name__)
+except PackageNotFoundError:
     # package is not installed
     pass
 
 
+mimetypes = None
+
+
 # Girder 3 is pinned to use pymongo < 4; its warnings aren't relevant until
 # that changes.
 warnings.filterwarnings('ignore', category=UserWarning, module='pymongo')
@@ -210,6 +220,33 @@ def handleRemoveFile(event):
             ImageItem().delete(item, [fileObj['_id']])
 
 
+def handleFinalizeUploadBefore(event):
+    """
+    When a file is uploaded, mark its mime type based on its extension if we
+    would otherwise just mark it as generic application/octet-stream.
+    """
+    fileObj = event.info['file']
+    if fileObj.get('mimeType', None) in {None, 'application/octet-stream'}:
+        global mimetypes
+
+        if not mimetypes:
+            import mimetypes
+
+            if not mimetypes.inited:
+                mimetypes.init()
+            # Augment the standard mimetypes with some additional values
+            for mimeType, ext, std in [
+                ('text/yaml', '.yaml', True),
+                ('text/yaml', '.yml', False),
+            ]:
+                if ext not in mimetypes.types_map:
+                    mimetypes.add_type(mimeType, ext, std)
+
+        alt = mimetypes.guess_type(fileObj.get('name', ''))[0]
+        if alt is not None:
+            fileObj['mimeType'] = alt
+
+
 # Validators
 
 @setting_utilities.validator({
@@ -280,6 +317,14 @@ def validateDefaultViewer(doc):
     doc['value'] = str(doc['value']).strip()
 
 
+@setting_utilities.validator(constants.PluginSettings.LARGE_IMAGE_CONFIG_FOLDER)
+def validateFolder(doc):
+    if not doc.get('value', None):
+        doc['value'] = None
+    else:
+        Folder().load(doc['value'], force=True, exc=True)
+
+
 # Defaults
 
 # Defaults that have fixed values can just be added to the system defaults
@@ -317,6 +362,7 @@ class LargeImagePlugin(GirderPlugin):
         curConfig = config.getConfig().get('large_image')
         for key, value in (curConfig or {}).items():
             large_image.config.setConfig(key, value)
+        addSystemEndpoints(info['apiRoot'])
 
         girder_tilesource.loadGirderTileSources()
         TilesItemResource(info['apiRoot'])
@@ -340,3 +386,4 @@ class LargeImagePlugin(GirderPlugin):
         events.bind('model.item.remove', 'large_image.removeThumbnails', removeThumbnails)
         events.bind('server_fuse.unmount', 'large_image', large_image.cache_util.cachesClear)
         events.bind('model.file.remove', 'large_image', handleRemoveFile)
+        events.bind('model.file.finalizeUpload.before', 'large_image', handleFinalizeUploadBefore)
