@@ -1,3 +1,5 @@
+import json
+
 import yaml
 
 from girder import logger
@@ -9,6 +11,9 @@ from girder.models.file import File
 from girder.models.folder import Folder
 from girder.models.group import Group
 from girder.models.item import Item
+from girder.models.setting import Setting
+
+from .. import constants
 
 
 def addSystemEndpoints(apiRoot):
@@ -19,6 +24,33 @@ def addSystemEndpoints(apiRoot):
     """
     apiRoot.folder.route('GET', (':id', 'yaml_config', ':name'), getYAMLConfigFile)
 
+    origItemFind = apiRoot.item._find
+    origFolderFind = apiRoot.folder._find
+
+    @boundHandler(apiRoot.item)
+    def altItemFind(self, folderId, text, name, limit, offset, sort, filters=None):
+        if sort and sort[0][0][0] == '[':
+            sort = json.loads(sort[0][0])
+        if filters is None and text and text.startswith('_filter_:'):
+            try:
+                filters = json.loads(text.split('_filter_:', 1)[1].strip())
+                text = None
+            except Exception as exc:
+                logger.warning('Failed to parse _filter_ from text field: %r', exc)
+        return origItemFind(folderId, text, name, limit, offset, sort, filters)
+
+    @boundHandler(apiRoot.item)
+    def altFolderFind(self, parentType, parentId, text, name, limit, offset, sort, filters=None):
+        if sort and sort[0][0][0] == '[':
+            sort = json.loads(sort[0][0])
+        return origFolderFind(parentType, parentId, text, name, limit, offset, sort, filters)
+
+    if not hasattr(origItemFind, '_origFunc'):
+        apiRoot.item._find = altItemFind
+        altItemFind._origFunc = origItemFind
+        apiRoot.folder._find = altFolderFind
+        altFolderFind._origFunc = origFolderFind
+
 
 def _mergeDictionaries(a, b):
     """
@@ -96,10 +128,11 @@ def adjustConfigForUser(config, user):
 )
 @boundHandler()
 def getYAMLConfigFile(self, folder, name):
+    addConfig = None
     user = self.getCurrentUser()
+    last = False
     while folder:
-        item = Item().findOne({'folderId': folder['_id'], 'name': name},
-                              user=user, level=AccessType.READ)
+        item = Item().findOne({'folderId': folder['_id'], 'name': name})
         if item:
             for file in Item().childFiles(item):
                 if file['size'] > 10 * 1024 ** 2:
@@ -110,13 +143,28 @@ def getYAMLConfigFile(self, folder, name):
                     # combine and adjust config values based on current user
                     if isinstance(config, dict) and 'access' in config or 'group' in config:
                         config = adjustConfigForUser(config, user)
-                    return config
+                    if addConfig and isinstance(config, dict):
+                        config = _mergeDictionaries(config, addConfig)
+                    if not isinstance(config, dict) or config.get('__inherit__') is not True:
+                        return config
+                    config.pop('__inherit__')
+                    addConfig = config
+        if last:
+            break
         if folder['parentCollection'] != 'folder':
-            if folder['name'] == '.config':
-                break
-            folder = Folder().findOne({
-                'parentId': folder['parentId'],
-                'parentCollection': folder['parentCollection'],
-                'name': '.config'})
+            if folder['name'] != '.config':
+                folder = Folder().findOne({
+                    'parentId': folder['parentId'],
+                    'parentCollection': folder['parentCollection'],
+                    'name': '.config'})
+            else:
+                last = 'setting'
+            if not folder or last == 'setting':
+                folderId = Setting().get(constants.PluginSettings.LARGE_IMAGE_CONFIG_FOLDER)
+                if not folderId:
+                    break
+                folder = Folder().load(folderId, force=True)
+                last = True
         else:
             folder = Folder().load(folder['parentId'], user=user, level=AccessType.READ)
+    return addConfig
